package Alien::SpiderMonkey;
$VERSION = '0.08';
1;

=head1 NAME

Alien::SpiderMonkey - Automated installation of JavaScript::SpiderMonkey

=head1 SYNOPSIS

There is no synopsis. All of the magic happens within C<Makefile.PL>.

=head1 DESCRIPTION

Alien::SpiderMonkey is my first try at making the installation of
JavaScript::SpiderMonkey automated. The installation of JavaScript::SpiderMonkey
is hard, because it relies on an external library, the SpiderMonkey Javascript
libary from Mozilla.org. The C<Makefile.PL> downloads that library and checks
that it builds. Then it downloads Javascript::SpiderMonkey and defers the
build process to it.

=head1 WARNINGS

The build and download process is very stupid and simple minded. There is
no checking whether there already is a suitable version of SpiderMonkey
installed somewhere on your system, or any elaborate downloading scheme.
Currently, the installation even requires a live network connection to
determine what the most recent version of SpiderMonkey is.

=head1 BUGS / ENHANCEMENTS

Currently, at least under Win32, the Spidermonkey DLL C<js32.dll> is copied 
into the directory where your Perl executable resides. This is ugly, but
makes the subsequent testing of JavaScript::SpiderMonkey easier. Fixes
for that ugly behaviour are very welcome.

Bug reports are welcome, but please include all relevant information
and try to include a fix as well. This is alpha software.

=cut

use strict;
use Config qw(%Config);
use Cwd;
use URI::URL;
use LWP::Simple;
use Net::FTP;
use File::Listing;
use vars qw($url $VERSION @ISA @EXPORT);
require Exporter;
@ISA = 'Exporter';
@EXPORT = qw(
	     download_spidermonkey
	     latest_spidermonkey
	     build_spidermonkey

	     download_js_spidermonkey
	    );

use vars qw($JS_DIR_URL $JS_LIB_TARGET $CPAN $CPAN_PATH $CPAN_MODULE);
$JS_DIR_URL = "ftp://sunsite.rediris.es/pub/mozilla.org/js/";
$JS_LIB_TARGET = 'jslib.tar.gz';

$CPAN = 'http://search.cpan.org/';
$CPAN_PATH = 'CPAN/authors/id/M/MS/MSCHILLI/';
$CPAN_MODULE = 'JavaScript-SpiderMonkey-0.08';

$VERSION=0.08;

{
  my $connection;

  sub ftp_connect {
    $url = URI::URL->new($JS_DIR_URL);
    die "Didn't find a Mozilla mirror. Set \$JS_DIR_URL" unless $JS_DIR_URL;
    $connection ||= do {
      print "Connecting to $JS_DIR_URL\n";
      my $host = $url->host();
      my $ftp = Net::FTP->new($host);
      die "Cannot connect to $host: ", $ftp->message
        unless $ftp;
      $ftp->login("anonymous",'-anonymous@') or
           die "Cannot login to $host: ", $ftp->message;
      $ftp->cwd($url->path) or
           die "Cannot change working directory on $host: ", $ftp->message;
      $ftp;
    };

    $connection;
  };
};

=head2 C<latest_spidermonkey>

Returns the filename of the latest version
of the SpiderMonkey library, as found on the mirror.

=cut

sub latest_spidermonkey {
  my $ftp = ftp_connect;
  my $candidate;
  foreach(File::Listing::parse_dir($ftp->dir())) {
       my ($name) = @$_;
       $candidate = $name if $name =~ /^js-1.*tar.gz$/;
  };
  $candidate;
};


=head2 C<download_spidermonkey [FILENAME]>

Downloads file C<FILENAME> or the latest version
into the local file C<jslib.tar.gz>. The host
and path used are stored in C<$JS_LIB_URL>.

=cut

sub download_spidermonkey {
  my ($candidate) = @_;
  my $target = 'jslib.tar.gz';
  $candidate ||= latest_spidermonkey();
  if (! -f $target) {
    my $ftp = ftp_connect;
    $ftp->binary();
    print "Downloading $candidate to $JS_LIB_TARGET\n";
    $ftp->get($candidate,$target);
    $ftp->quit;
  } else {
    print "$candidate already exists, skipping download\n";
  };

  return $target;
};

=head2 C<build_spidermonkey DIR>

Builds the SpiderMonkey library.

=cut

sub build_spidermonkey {
  my $dir = shift || "js/src";
  my $old_dir = cwd;

  chdir $dir
    or die "Couldn't chdir to $dir : $!";

  # as per Readme.html
  if ($^O =~ /win32/i) {
    system( 'nmake','-f', 'js.mak', 'CFG=js - Win32 Release' ) == 0
      or die "nmake returned status $?/$!";
    system( 'nmake','-f', 'js.mak', 'CFG=fdlibm - Win32 Release' ) == 0
      or die "nmake returned status $?/$!";
  } else {
    system($Config{make}, '-f', 'Makefile.ref' ) == 0
      or die "$Config{make} returned status $?/$!";
  };

  chdir $old_dir
    or warn "Couldn't restore directory '$old_dir': $!";
};

=head2 C<download_js_spidermonkey>

Downloads the module JavaScript::SpiderMonkey
from CPAN.

=cut

sub download_js_spidermonkey {
  my $filename = "$CPAN_MODULE.tar.gz";
  mirror("$CPAN$CPAN_PATH$filename",$filename);
  -f $filename
    or die "$filename did not download from $CPAN";
  $filename
};

1;
